; *==============================================================================
; *
; * - TI-Vibe for TI-89/92+/V200/Titanium -
; * Copyright (C) 2006 Olivier Armand  <olivier d0t calc at gmail d0t com>
; * All rights reserved.
; *
; * Redistribution and use in source and binary forms, with or without
; * modification, are permitted provided that the following conditions
; * are met:
; * 1. Redistributions of source code must retain the above copyright
; *    notice, this list of conditions and the following disclaimer.
; * 2. Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the
; *    documentation and/or other materials provided with the distribution.
; *
; * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
; * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
; * ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS BE LIABLE
; * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
; * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
; * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
; * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
; * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; *==============================================================================

	include	"OS.h"
	xdef	_nostub
	xdef	_ti89

degrees_num	equ	128	; 7-bit sound, we can't have more on TI68k

rcall	macro
	move.l	\1*4(a5),a0
	jsr	(a0)
	endm

rcall_var	macro
	move.l	\1*4(a5),\2
	endm

Main:	
	movem.l	d3-d7/a2-a6,-(a7)
	move.w	#$700,d0
	trap	#1
	lea	$60000E,a6	; keep - direct link port access
	move.l	($C8).w,a5	; keep
	bset.b	#6,-2(a6)	; disable byte sender/receiver
	link 	a4,#-4		; frame pointer
	clr.l	(a4)		; file_table_ptr: array of size_of_file-1.w, start_of_file_ptr --- 0.w terminated

;----------------------------------------------------------

GetSymstrArg:
	rcall_var top_estack,a3
	move.l	(a3),a3
	pea	(a3)
	rcall	RemainingArgCnt
	addq.w	#4,a7
	subq.w	#1,d0
	bne	Usage
	cmp.b	#$2D,(a3)	; 2D = STR_TAG
	bne	Usage
	subq.w	#1,a3		; a2 = FolderNameSymPtr

LoadFiles:
	pea	(100*(2+4)).w	; at most 100 files, ie 100 * (1 size-1.w + 1 beginning_ptr)
	rcall	HeapAllocPtr
	move.l	a0,(a4)		; frame pointer
	beq	NoMem
	move.l	a0,a2
	move.w	#1,(a7)		; FO_SINGLE_FOLDER
	pea	(a3)		; symptr
	move.l	a0,a3		; keep - file table pointer
	rcall	SymFindFirst
	addq.w	#8,a7
\LoopFiles:
	move.l 	a0,d0
	beq	\EndLoopFiles
	move.w	12(a0),-(a7)
	rcall	HeapDeref
	addq.w	#2,a7
	move.w	(a0)+,d0	; size of the variable
	subq.w	#6+1,d0		; len(\x00WAV\x00\xF8), +1 for the dbra when it will be played
	move.w	d0,(a2)+	; size of the actual data
	move.l	a0,(a2)+	; beginning of the file
	rcall	SymFindNext
	bra	\LoopFiles
\EndLoopFiles:
	cmp.l	a2,a3		; current ptr = file table ptr
	bne	\FilesFound
	pea	SoundFilesNotFoundStr(pc)
	bra	Abort
\FilesFound:
	clr.w	(a2)+		; null terminated

;----------------------------------------------------------

Play:
	pea	NowPlayingStr(pc)
	rcall	ST_helpMsg
	addq.w	#4,a7

	moveq	#%11,d0		; white and red wires for the direct link port access - falling edge
	moveq	#0,d6		; rising edge
	lea	FallingEdge(pc),a2
	move.w	(a2),d4
	move.w	#$4E71,d3	; 'nop' opcode, to restore the PWM retoune
	moveq	#0,d1		; because of the d1.l further
	lea	$60001B,a2
	clr.b	-3(a2)		; $600018, kbd mask: none
	moveq	#-1,d5		; if $60001B = d0, no key has been pressed
\WaitNoKeyPressed:
	cmp.b	(a2),d5
	bne	\WaitNoKeyPressed ; wait for "enter" to be release
	dbra	d5,*		; Wait to avoid the bounce with some enter keys. Before and after, d5=-1
	
	bra	 NextFile

; The following loop (from PlayFilesLoop to Played) should be as optimized as possible to get the best sample rate.
PlayFilesLoop:
	move.l	(a3)+,a0	; ptr to the beginning of the file
PlayAFile:
	move.b	(a0)+,d1	; next wave byte - already multiplied by 2 by tivibegen to be an offset for the nop instrs
	lea	PWMNops(pc,d1.l),a1
	move.w	d4,(a1)		; bclr 

; Pulse Width Modulation: generates a pulse for the current sample.
; We must be able to create degrees_num different widths of pulse.
; The precision is the number of cycles of a nop instruction (2).
	move.b	d0,(a6)	; rising edge of the pulse for the 2 channels
	; nops. the instruction 'FallingEdge' will be written somewhere here,
	; for each sample, then restored to a nop.
PWMNops	dcb.w	degrees_num,$4E71

	move.w	d3,(a1)		; nop - restore
	cmp.b	(a2),d5
	dbne	d2,PlayAFile	; key pressed/end of file?
	bne	Played		; key pressed
NextFile:
	move.w	(a3)+,d2	; size-1 of the next file [dbra index] (a3 : file array ptr)
	bne	PlayFilesLoop	; not the last file

Played:
	rcall	ST_eraseHelp
\WaitNoKeyPressed:
	cmp.b	(a2),d5
	bne	\WaitNoKeyPressed ; wait for the key to be release
	
;----------------------------------------------------------

Clean:
	move.l	(a4),-(a7)
	beq.s	\NoFileTable
	rcall	HeapFreePtr
\NoFileTable:
	addq.w	#4,a7

Quit:
	unlk	a4
	bclr.b	#6,-2(a6)	; enable byte sender/receiver
	moveq	#0,d0
	trap	#1	
	movem.l	(a7)+,d3-d7/a2-a6
	rts
	
;----------------------------------------------------------

Usage:
	pea	UsageStr(pc)
	bra	Abort

NoMem:
	addq.w	#4,a7
	pea	MemoryStr(pc)
;	bra	Abort

Abort:
	rcall	ST_helpMsg
	addq.w	#4,a7
	bra	Clean

;----------------------------------------------------------

FallingEdge:
	move.b	d6,(a6)

;----------------------------------------------------------

UsageStr		dc.b	"Usage: tivibe('foldername')",0
MemoryStr		dc.b	"Not enough memory.",0
SoundFilesNotFoundStr	dc.b	"Sound files not found.",0
NowPlayingStr		dc.b	"Now playing... Press any key to abort.",0
